==> plone.app.contenttypes-3.0.3.tar.gz: plone.app.contenttypes-3.0.3/plone/app/contenttypes/tests/test_link.py <==
from datetime import datetime
from plone.app.contenttypes.interfaces import ILink
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.z3cform.converters import LinkWidgetDataConverter
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.app.z3cform.widget import LinkWidget
from plone.dexterity.interfaces import IDexterityFTI
from plone.testing.zope import Browser
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.event import notify
from zope.interface import alsoProvides
from zope.schema import TextLine
from zope.traversing.interfaces import BeforeTraverseEvent

import unittest


class LinkIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="Link")
        schema = fti.lookupSchema()
        self.assertTrue(schema.getName().endswith("_0_Link"))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="Link")
        self.assertNotEqual(None, fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="Link")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(ILink.providedBy(new_object))

    def test_adding(self):
        self.portal.invokeFactory("Link", "doc1")
        self.assertTrue(ILink.providedBy(self.portal["doc1"]))


class LinkViewIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        self.response = self.request.response
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Link", "link")
        link = self.portal["link"]
        link.title = "My Link"
        link.description = "This is my link."
        self.link = link
        self.request.set("URL", link.absolute_url())
        self.request.set("ACTUAL_URL", link.absolute_url())
        alsoProvides(self.request, IPloneFormLayer)

        # setup manually the correct browserlayer, see:
        # https://dev.plone.org/ticket/11673
        notify(BeforeTraverseEvent(self.portal, self.request))

    def test_link_redirect_view(self):
        view = self.link.restrictedTraverse("@@view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)
        self.assertTrue("My Link" in view())
        self.assertTrue("This is my link." in view())

    def test_link_redirect_view_external_url(self):
        self.link.remoteUrl = "http://www.plone.org"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect("http://www.plone.org")

    def test_link_redirect_view_absolute_path(self):
        self.link.remoteUrl = "/plone/my-folder/my-item"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect("http://nohost/plone/my-folder/my-item")

    def test_link_redirect_view_relative_path(self):
        self.link.remoteUrl = "../my-item"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        # The following URL will be redirected to:
        # "http://nohost/plone/my-item"
        self._assert_redirect("http://nohost/plone/link/../my-item")

    def test_link_redirect_view_path_with_variable(self):
        self.link.remoteUrl = "${navigation_root_url}/my-folder/my-item"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect("http://nohost/plone/my-folder/my-item")

        # Should give the same result with ${portal_url}
        self.link.remoteUrl = "${portal_url}/my-folder/my-item"
        self.assertTrue(view())
        self._assert_redirect("http://nohost/plone/my-folder/my-item")

    def test_link_redirect_view_path_with_variable_and_parameters(self):
        # https://github.com/plone/plone.app.contenttypes/issues/457
        self.link.remoteUrl = "${portal_url}/@@search?SearchableText=Plone"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect(
            "http://nohost/plone/@@search?SearchableText=Plone",
        )

    def test_mailto_type(self):
        self.link.remoteUrl = "mailto:stress@test.us"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="mailto:stress@test.us"', rendered)
        self._assert_response_OK()

    def test_tel_type(self):
        self.link.remoteUrl = "tel:123"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="tel:123"', rendered)
        self._assert_response_OK()

    def test_callto_type(self):
        self.link.remoteUrl = "callto:123"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="callto:123"', rendered)
        self._assert_response_OK()

    def test_webdav_type(self):
        self.link.remoteUrl = "webdav://web.site/resource"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="webdav://web.site/resource"', rendered)
        self._assert_response_OK()

    def test_caldav_type(self):
        self.link.remoteUrl = "caldav://calendar.site/resource"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="caldav://calendar.site/resource"', rendered)
        self._assert_response_OK()

    def test_file_type(self):
        self.link.remoteUrl = "file:///some/file/on/your/system"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        self.assertTrue(view())
        self._assert_redirect(self.link.remoteUrl)

    def test_ftp_type(self):
        self.link.remoteUrl = "ftp://thereIsNoSuchDomain.isThere{}".format(
            datetime.now().isoformat()
        )
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        self.assertTrue(view())
        self._assert_redirect(self.link.remoteUrl)

    def _publish(self, obj):
        portal_workflow = getToolByName(self.portal, "portal_workflow")
        portal_workflow.doActionFor(obj, "publish")

    def _assert_redirect(self, url):
        self.assertEqual(self.response.status, 302)
        self.assertEqual(self.response.headers["location"], url)

    def _assert_response_OK(self):
        self.assertEqual(self.response.status, 200)

    def _get_link_redirect_view(self, obj):
        return getMultiAdapter((obj, self.request), name="link_redirect_view")


class LinkFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_add_link(self):
        self.browser.open(self.portal_url)
        self.browser.getLink("Link").click()
        self.browser.getControl(name="form.widgets.IDublinCore.title").value = "My link"
        self.browser.getControl(
            name="form.widgets.IDublinCore.description"
        ).value = "This is my link."
        self.browser.getControl(
            name="form.widgets.IShortName.id"
        ).value = "my-special-link"
        self.browser.getControl(
            name="form.widgets.remoteUrl.external"
        ).value = "https://plone.org"
        self.browser.getControl("Save").click()

        self.assertTrue(self.browser.url.endswith("my-special-link/view"))
        self.assertTrue("My link" in self.browser.contents)
        self.assertTrue("This is my link" in self.browser.contents)


class LinkWidgetIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    default_result = {
        "internal": "",
        "external": "",
        "email": "",
        "email_subject": "",
    }

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        self.response = self.request.response
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        lid = self.portal.invokeFactory(
            "Link", "link", title="My Link", description="This is my link."
        )
        self.link = self.portal[lid]
        self.link_field = TextLine()
        self.widget = LinkWidget(self.request)

    def test_dc_towidget_external(self):
        url = "http://www.example.org"
        self.link.remoteUrl = url

        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["external"] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_internal(self):
        self.portal.invokeFactory(
            "Document", "doc1", title="A document", description="This is a document."
        )
        doc1 = self.portal["doc1"]
        url = doc1.absolute_url()
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["internal"] = IUUID(doc1)
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_mail(self):
        url = "mailto:foo@.example.org"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["email"] = url[7:]  # mailto is cut
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_mail_subject(self):
        url = "mailto:foo@.example.org?subject=A subject"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["email"] = "foo@.example.org"
        expected["email_subject"] = "A subject"
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_illegal(self):
        url = "foo"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["external"] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_var(self):
        url = "${portal_url}/foo"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["external"] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_var_replacement_in_view(self):
        view = getMultiAdapter((self.link, self.request), name="link_redirect_view")

        self.link.remoteUrl = "${portal_url}"
        self.assertEqual(view.url(), "/plone")
        self.assertEqual(view.absolute_target_url(), "http://nohost/plone")

        self.link.remoteUrl = "${navigation_root_url}"
        self.assertEqual(view.url(), "/plone")
        self.assertEqual(view.absolute_target_url(), "http://nohost/plone")

    def test_resolve_uid_to_absolute_target(self):
        view = getMultiAdapter((self.link, self.request), name="link_redirect_view")

        self.portal.invokeFactory(
            "Document", "doc1", title="A document", description="This is a document."
        )
        doc1 = self.portal["doc1"]
        uid = IUUID(doc1)
        self.link.remoteUrl = f"${{portal_url}}/resolveuid/{uid}"
        self.assertEqual(view.absolute_target_url(), "http://nohost/doc1")

==> plone.app.contenttypes-3.0.6.tar.gz: plone.app.contenttypes-3.0.6/plone/app/contenttypes/tests/test_link.py <==
from datetime import datetime
from plone.app.contenttypes.browser.link_redirect_view import normalize_uid_from_path
from plone.app.contenttypes.interfaces import ILink
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone.app.contenttypes.testing import (  # noqa
    PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.z3cform.converters import LinkWidgetDataConverter
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.app.z3cform.widget import LinkWidget
from plone.dexterity.interfaces import IDexterityFTI
from plone.testing.zope import Browser
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.event import notify
from zope.interface import alsoProvides
from zope.schema import TextLine
from zope.traversing.interfaces import BeforeTraverseEvent

import unittest


class LinkIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="Link")
        schema = fti.lookupSchema()
        self.assertTrue(schema.getName().endswith("_0_Link"))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="Link")
        self.assertNotEqual(None, fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="Link")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(ILink.providedBy(new_object))

    def test_adding(self):
        self.portal.invokeFactory("Link", "doc1")
        self.assertTrue(ILink.providedBy(self.portal["doc1"]))


class LinkViewIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        self.response = self.request.response
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Link", "link")
        link = self.portal["link"]
        link.title = "My Link"
        link.description = "This is my link."
        self.link = link
        self.request.set("URL", link.absolute_url())
        self.request.set("ACTUAL_URL", link.absolute_url())
        alsoProvides(self.request, IPloneFormLayer)

        # setup manually the correct browserlayer, see:
        # https://dev.plone.org/ticket/11673
        notify(BeforeTraverseEvent(self.portal, self.request))

    def test_link_redirect_view(self):
        view = self.link.restrictedTraverse("@@view")
        self.assertTrue(view())
        self.assertEqual(view.request.response.status, 200)
        self.assertTrue("My Link" in view())
        self.assertTrue("This is my link." in view())

    def test_link_redirect_view_external_url(self):
        self.link.remoteUrl = "http://www.plone.org"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect("http://www.plone.org")

    def test_link_redirect_view_absolute_path(self):
        self.link.remoteUrl = "/plone/my-folder/my-item"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect("http://nohost/plone/my-folder/my-item")

    def test_link_redirect_view_relative_path(self):
        self.link.remoteUrl = "../my-item"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        # The following URL will be redirected to:
        # "http://nohost/plone/my-item"
        self._assert_redirect("http://nohost/plone/link/../my-item")

    def test_link_redirect_view_path_with_variable(self):
        self.link.remoteUrl = "${navigation_root_url}/my-folder/my-item"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect("http://nohost/plone/my-folder/my-item")

        # Should give the same result with ${portal_url}
        self.link.remoteUrl = "${portal_url}/my-folder/my-item"
        self.assertTrue(view())
        self._assert_redirect("http://nohost/plone/my-folder/my-item")

    def test_link_redirect_view_path_with_variable_and_parameters(self):
        # https://github.com/plone/plone.app.contenttypes/issues/457
        self.link.remoteUrl = "${portal_url}/@@search?SearchableText=Plone"
        self._publish(self.link)
        view = self._get_link_redirect_view(self.link)

        # As manager: do not redirect
        self.assertTrue(view())
        self.assertEqual(self.response.status, 200)

        # As anonymous: redirect
        logout()
        self.assertTrue(view())
        self._assert_redirect(
            "http://nohost/plone/@@search?SearchableText=Plone",
        )

    def test_mailto_type(self):
        self.link.remoteUrl = "mailto:stress@test.us"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="mailto:stress@test.us"', rendered)
        self._assert_response_OK()

    def test_tel_type(self):
        self.link.remoteUrl = "tel:123"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="tel:123"', rendered)
        self._assert_response_OK()

    def test_callto_type(self):
        self.link.remoteUrl = "callto:123"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="callto:123"', rendered)
        self._assert_response_OK()

    def test_webdav_type(self):
        self.link.remoteUrl = "webdav://web.site/resource"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="webdav://web.site/resource"', rendered)
        self._assert_response_OK()

    def test_caldav_type(self):
        self.link.remoteUrl = "caldav://calendar.site/resource"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        rendered = view()
        self.assertIn('href="caldav://calendar.site/resource"', rendered)
        self._assert_response_OK()

    def test_file_type(self):
        self.link.remoteUrl = "file:///some/file/on/your/system"
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        self.assertTrue(view())
        self._assert_redirect(self.link.remoteUrl)

    def test_ftp_type(self):
        self.link.remoteUrl = "ftp://thereIsNoSuchDomain.isThere{}".format(
            datetime.now().isoformat()
        )
        view = self._get_link_redirect_view(self.link)
        self._publish(self.link)
        logout()
        self.assertTrue(view())
        self._assert_redirect(self.link.remoteUrl)

    def test_normalize_uid_from_path(self):
        url = None
        uid, fragment = normalize_uid_from_path(url)
        self.assertIsNone(uid)
        self.assertIsNone(fragment)

        url = "http://nohost"
        uid, fragment = normalize_uid_from_path(url)
        self.assertIsNone(uid)
        self.assertIsNone(fragment)

        url = "http://nohost/test1"
        uid, fragment = normalize_uid_from_path(url)
        self.assertIsNone(uid)
        self.assertIsNone(fragment)

        url = "http://nohost/test1/resolveuid"
        uid, fragment = normalize_uid_from_path(url)
        self.assertIsNone(uid)
        self.assertIsNone(fragment)

        url = "http://nohost/test1/resolveuid/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertIsNone(uid)
        self.assertIsNone(fragment)

        url = "http://nohost/test1/resolveuid123/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertIsNone(uid)
        self.assertIsNone(fragment)

        url = "http://nohost/test1/resolveuid/123"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "http://nohost/test1/resolveuid/123#my-id"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertEqual(fragment, "#my-id")

        url = "http://nohost/test1/resolveuid/123/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "http://nohost/test1/resolveuid/123#my-id/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertEqual(fragment, "#my-id")

        url = "http://nohost/test1/resolveuid/123/test"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "resolveuid/123"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "resolveuid/123#my-id"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertEqual(fragment, "#my-id")

        url = "resolveuid/123/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "resolveuid/123#my-id/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertEqual(fragment, "#my-id")

        url = "resolveuid/123/abc"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "/resolveuid/123"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "/resolveuid/123#my-id"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertEqual(fragment, "#my-id")

        url = "/resolveuid/123/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "/resolveuid/123#my-id/"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertEqual(fragment, "#my-id")

        url = "/resolveuid/123/abc"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertIsNone(fragment)

        url = "/resolveUid/123#my-id"
        uid, fragment = normalize_uid_from_path(url)
        self.assertEqual(uid, "123")
        self.assertEqual(fragment, "#my-id")

    def _publish(self, obj):
        portal_workflow = getToolByName(self.portal, "portal_workflow")
        portal_workflow.doActionFor(obj, "publish")

    def _assert_redirect(self, url):
        self.assertEqual(self.response.status, 302)
        self.assertEqual(self.response.headers["location"], url)

    def _assert_response_OK(self):
        self.assertEqual(self.response.status, 200)

    def _get_link_redirect_view(self, obj):
        return getMultiAdapter((obj, self.request), name="link_redirect_view")


class LinkFunctionalTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization",
            "Basic {}:{}".format(
                SITE_OWNER_NAME,
                SITE_OWNER_PASSWORD,
            ),
        )

    def test_add_link(self):
        self.browser.open(self.portal_url)
        self.browser.getLink("Link").click()
        self.browser.getControl(name="form.widgets.IDublinCore.title").value = "My link"
        self.browser.getControl(name="form.widgets.IDublinCore.description").value = (
            "This is my link."
        )
        self.browser.getControl(name="form.widgets.IShortName.id").value = (
            "my-special-link"
        )
        self.browser.getControl(name="form.widgets.remoteUrl.external").value = (
            "https://plone.org"
        )
        self.browser.getControl("Save").click()

        self.assertTrue(self.browser.url.endswith("my-special-link/view"))
        self.assertTrue("My link" in self.browser.contents)
        self.assertTrue("This is my link" in self.browser.contents)


class LinkWidgetIntegrationTest(unittest.TestCase):
    layer = PLONE_APP_CONTENTTYPES_INTEGRATION_TESTING

    default_result = {
        "internal": "",
        "external": "",
        "email": "",
        "email_subject": "",
    }

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.request["ACTUAL_URL"] = self.portal.absolute_url()
        self.response = self.request.response
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        lid = self.portal.invokeFactory(
            "Link", "link", title="My Link", description="This is my link."
        )
        self.link = self.portal[lid]
        self.link_field = TextLine()
        self.widget = LinkWidget(self.request)

    def test_dc_towidget_external(self):
        url = "http://www.example.org"
        self.link.remoteUrl = url

        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["external"] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_internal(self):
        self.portal.invokeFactory(
            "Document", "doc1", title="A document", description="This is a document."
        )
        doc1 = self.portal["doc1"]
        url = doc1.absolute_url()
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["internal"] = IUUID(doc1)
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_mail(self):
        url = "mailto:foo@.example.org"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["email"] = url[7:]  # mailto is cut
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_towidget_mail_subject(self):
        url = "mailto:foo@.example.org?subject=A subject"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["email"] = "foo@.example.org"
        expected["email_subject"] = "A subject"
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_illegal(self):
        url = "foo"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["external"] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_dc_var(self):
        url = "${portal_url}/foo"
        converter = LinkWidgetDataConverter(self.link_field, self.widget)
        expected = self.default_result.copy()
        expected["external"] = url
        self.assertEqual(converter.toWidgetValue(url), expected)

    def test_var_replacement_in_view(self):
        view = getMultiAdapter((self.link, self.request), name="link_redirect_view")

        self.link.remoteUrl = "${portal_url}"
        self.assertEqual(view.url(), "/plone")
        self.assertEqual(view.absolute_target_url(), "http://nohost/plone")

        self.link.remoteUrl = "${navigation_root_url}"
        self.assertEqual(view.url(), "/plone")
        self.assertEqual(view.absolute_target_url(), "http://nohost/plone")

    def test_resolve_uid_to_absolute_target(self):
        view = getMultiAdapter((self.link, self.request), name="link_redirect_view")

        self.portal.invokeFactory(
            "Document", "doc1", title="A document", description="This is a document."
        )
        doc1 = self.portal["doc1"]
        uid = IUUID(doc1)

        portal_state = getMultiAdapter(
            (self.link, self.request), name="plone_portal_state"
        )
        portal_url = portal_state.portal_url()

        # check an internal link
        self.link.remoteUrl = f"${{portal_url}}/resolveuid/{uid}"
        self.assertEqual(view.absolute_target_url(), f"{portal_url}/doc1")

        # check an internal link with fragment
        self.link.remoteUrl = f"${{portal_url}}/resolveuid/{uid}#autotoc-item-autotoc-1"
        self.assertEqual(
            view.absolute_target_url(), f"{portal_url}/doc1#autotoc-item-autotoc-1"
        )

        # check not resolvable uid
        self.link.remoteUrl = "/resolveuid/abc123"
        self.assertEqual(view.absolute_target_url(), "/resolveuid/abc123")
